"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var THREE = _interopRequireWildcard(require("three"));

var _Coordinates = _interopRequireWildcard(require("./Coordinates"));

var _Projection = _interopRequireDefault(require("./Projection"));

/**
 * Extent is a SIG-area (so 2D)
 * It can use explicit coordinates (e.g: lon/lat) or implicit (WMTS coordinates)
 */
var CARDINAL = {
  WEST: 0,
  EAST: 1,
  SOUTH: 2,
  NORTH: 3
};

var _dim = new THREE.Vector2();

var _dim2 = new THREE.Vector2();

var r = {
  row: 0,
  col: 0,
  invDiff: 0
};

function _rowColfromParent(extent, zoom) {
  var diffLevel = extent.zoom - zoom;
  var diff = Math.pow(2, diffLevel);
  r.invDiff = 1 / diff;
  r.row = (extent.row - extent.row % diff) * r.invDiff;
  r.col = (extent.col - extent.col % diff) * r.invDiff;
  return r;
}

var _extent;

var cardinals = [];
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0));
cardinals.push(new _Coordinates.default('EPSG:4326', 0, 0, 0, 0)); // EPSG:3857
// WGS84 bounds [-180.0, -85.06, 180.0, 85.06] (https://epsg.io/3857)
// Warning, some tiled source don't exactly match the same bound
// It should be taken into account

var _c = new _Coordinates.default('EPSG:4326', 180, 85.06);

_c.as('EPSG:3857', _c); // Get bound dimension in 'EPSG:3857'


var sizeX = _c._values[0] * 2;
var sizeY = _c._values[1] * 2;

var Extent =
/*#__PURE__*/
function () {
  /**
   * Extent is geographical bounding rectangle defined by 4 limits: west, east, south and north.
   * If crs is tiled projection (WMTS or TMS), the extent is defined by zoom, row and column.
   *
   * @param {String} crs projection of limit values.
   * @param {number|Array.<number>|Coordinates|Object} v0 west value, zoom value, Array of values [west, east, south and north],
   * Coordinates of west-south corner or object {west, east, south and north}
   * @param {number|Coordinates} [v1] east value, row value or Coordinates of east-north corner
   * @param {number} [v2] south value or column value
   * @param {number} [v3] north value
   */
  function Extent(crs, v0, v1, v2, v3) {
    (0, _classCallCheck2.default)(this, Extent);
    this.crs = crs;

    if (this.isTiledCrs()) {
      if (v0 !== undefined) {
        this.zoom = v0;
        this.row = v1;
        this.col = v2;

        if (this.zoom < 0) {
          throw new Error('invlid WTMS values');
        }
      } else {
        throw new Error('Unsupported constructor args');
      }
    } else if (v0 instanceof _Coordinates.default) {
      // seem never used
      this._values = new Float64Array([v0._values[0], v1._values[0], v0._values[1], v1._values[1]]);
    } else if (v0 && v0.west !== undefined) {
      this._values = new Float64Array([v0.west, v0.east, v0.south, v0.north]);
    } else if (v0.length == 4) {
      this._values = new Float64Array(v0);
    } else if (v0 !== undefined) {
      this._values = new Float64Array([v0, v1, v2, v3]);
    } else {
      throw new Error('Unsupported constructor args');
    }
  }
  /**
   * Clone this extent
   * @return {Extent} cloned extent
   */


  (0, _createClass2.default)(Extent, [{
    key: "clone",
    value: function clone() {
      if (this.isTiledCrs()) {
        return new Extent(this.crs, this.zoom, this.row, this.col);
      } else {
        return new Extent(this.crs, this._values);
      }
    }
    /**
     * Return true is tiled Extent (WGS84, PM)
     * @return {boolean}
     */

  }, {
    key: "isTiledCrs",
    value: function isTiledCrs() {
      return this.crs.indexOf('WMTS:') == 0 || this.crs == 'TMS';
    }
    /**
     * Convert Extent to the specified projection.
     * @param {string} crs the projection of destination.
     * @param {Extent} target copy the destination to target.
     * @return {Extent}
     */

  }, {
    key: "as",
    value: function as(crs, target) {
      (0, _Coordinates.assertCrsIsValid)(crs);

      if (this.isTiledCrs()) {
        if (this.crs == 'WMTS:PM' || this.crs == 'TMS') {
          if (!target) {
            target = new Extent('EPSG:4326', [0, 0, 0, 0]);
          } // Convert this to the requested crs by using 4326 as an intermediate state.


          var nbCol = Math.pow(2, this.zoom);
          var nbRow = nbCol;
          var sizeRow = 1.0 / nbRow; // convert row PM to Y PM

          var Yn = 1 - sizeRow * (nbRow - this.row);
          var Ys = Yn + sizeRow; // convert to EPSG:3857

          if (crs == 'EPSG:3857') {
            var west = (0.5 - sizeRow * (nbCol - this.col)) * sizeX;
            target.set(west, west + sizeRow * sizeX, (0.5 - Ys) * sizeY, (0.5 - Yn) * sizeY);
            target.crs = 'EPSG:3857';
            return target.as(crs, target);
          } else {
            var size = 360 / nbCol; // convert Y PM to latitude EPSG:4326 degree

            var _north = _Projection.default.YToWGS84(Yn);

            var _south = _Projection.default.YToWGS84(Ys); // convert column PM to longitude EPSG:4326 degree


            var _west = 180 - size * (nbCol - this.col);

            target.set(_west, _west + size, _south, _north);
            target.crs = 'EPSG:4326';

            if (crs == 'EPSG:4326') {
              return target;
            } else {
              // convert in new crs
              return target.as(crs, target);
            }
          }
        } else if (this.crs == 'WMTS:WGS84G' && crs == 'EPSG:4326') {
          if (!target) {
            target = new Extent('EPSG:4326', [0, 0, 0, 0]);
          }

          var _nbRow = Math.pow(2, this.zoom);

          var _size = 180 / _nbRow;

          var _north2 = _size * (_nbRow - this.row) - 90;

          var _south2 = _size * (_nbRow - (this.row + 1)) - 90;

          var _west2 = 180 - _size * (2 * _nbRow - this.col);

          var _east2 = 180 - _size * (2 * _nbRow - (this.col + 1));

          target.set(_west2, _east2, _south2, _north2);
          target.crs = crs;
          return target;
        } else {
          throw new Error('Unsupported yet');
        }
      }

      if (!target) {
        target = new Extent('EPSG:4326', [0, 0, 0, 0]);
      }

      if (this.crs != crs && !((0, _Coordinates.is4326)(this.crs) && (0, _Coordinates.is4326)(crs))) {
        // Compute min/max in x/y by projecting 8 cardinal points,
        // and then taking the min/max of each coordinates.
        var center = this.center(_c);
        cardinals[0].set(this.crs, this.west, this.north);
        cardinals[1].set(this.crs, center._values[0], this.north);
        cardinals[2].set(this.crs, this.east, this.north);
        cardinals[3].set(this.crs, this.east, center._values[1]);
        cardinals[4].set(this.crs, this.east, this.south);
        cardinals[5].set(this.crs, center._values[0], this.south);
        cardinals[6].set(this.crs, this.west, this.south);
        cardinals[7].set(this.crs, this.west, center._values[1]);

        var _north3 = -Infinity;

        var _south3 = Infinity;

        var _east3 = -Infinity;

        var _west3 = Infinity; // loop over the coordinates

        for (var i = 0; i < cardinals.length; i++) {
          // convert the coordinate.
          cardinals[i].as(crs, _c);
          _north3 = Math.max(_north3, _c._values[1]);
          _south3 = Math.min(_south3, _c._values[1]);
          _east3 = Math.max(_east3, _c._values[0]);
          _west3 = Math.min(_west3, _c._values[0]);
        }

        target.set(_west3, _east3, _south3, _north3);
        target.crs = crs;
        return target;
      }

      target.crs = crs;
      target.set(this.west, this.east, this.south, this.north);
      return target;
    }
    /**
     * Return the west value
     * @return {number}
     */

  }, {
    key: "center",

    /**
     * Return the center of Extent
     * @param {Coordinates} target copy the center to the target.
     * @return {Coordinates}
     */
    value: function center(target) {
      if (this.isTiledCrs()) {
        throw new Error('Invalid operation for WMTS bbox');
      }

      this.dimensions(_dim);

      if (target) {
        target.set(this.crs, this._values[0] + _dim.x * 0.5, this._values[2] + _dim.y * 0.5);
      } else {
        target = new _Coordinates.default(this.crs, this._values[0] + _dim.x * 0.5, this._values[2] + _dim.y * 0.5);
      }

      return target;
    }
    /**
    * Returns the dimension of the extent, in a <code>THREE.Vector2</code>.
    *
    * @param {THREE.Vector2} [target] - The target to assign the result in.
    *
    * @return {THREE.Vector2}
    */

  }, {
    key: "dimensions",
    value: function dimensions() {
      var target = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new THREE.Vector2();
      target.x = Math.abs(this.east - this.west);
      target.y = Math.abs(this.north - this.south);
      return target;
    }
    /**
     * Return true if <code>coord</code> is inside the bounding box.
     *
     * @param {Coordinates} coord
     * @param {number} [epsilon=0] - to take into account when comparing to the
     * point.
     *
     * @return {boolean}
     */

  }, {
    key: "isPointInside",
    value: function isPointInside(coord) {
      var epsilon = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var c = this.crs == coord.crs ? coord : coord.as(this.crs, _c); // TODO this ignores altitude

      if ((0, _Coordinates.crsIsGeographic)(this.crs)) {
        return c.longitude() <= this.east + epsilon && c.longitude() >= this.west - epsilon && c.latitude() <= this.north + epsilon && c.latitude() >= this.south - epsilon;
      } else {
        return c.x() <= this.east + epsilon && c.x() >= this.west - epsilon && c.y() <= this.north + epsilon && c.y() >= this.south - epsilon;
      }
    }
    /**
     * Return true if <code>extent</code> is inside this extent.
     *
     * @param {Extent} extent the extent to check
     * @param {number} epsilon to take into account when comparing to the
     * point.
     *
     * @return {boolean}
     */

  }, {
    key: "isInside",
    value: function isInside(extent, epsilon) {
      if (this.isTiledCrs()) {
        if (this.zoom == extent.zoom) {
          return this.row == extent.row && this.col == extent.col;
        } else if (this.zoom < extent.zoom) {
          return false;
        } else {
          _rowColfromParent(this, extent.zoom);

          return r.row == extent.row && r.col == extent.col;
        }
      } else {
        extent.as(this.crs, _extent);
        epsilon = epsilon == undefined ? (0, _Coordinates.reasonnableEpsilonForCRS)(this.crs) : epsilon;
        return this.east - _extent.east <= epsilon && _extent.west - this.west <= epsilon && this.north - _extent.north <= epsilon && _extent.south - this.south <= epsilon;
      }
    }
    /**
     * Return the translation and scale to transform this extent to input extent.
     *
     * @param {Extent} extent input extent
     * @param {THREE.Vector4} target copy the result to target.
     * @return {THREE.Vector4} {x: translation on west-east, y: translation on south-north, z: scale on west-east, w: scale on south-north}
     */

  }, {
    key: "offsetToParent",
    value: function offsetToParent(extent) {
      var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new THREE.Vector4();

      if (this.crs != extent.crs) {
        throw new Error('unsupported mix');
      }

      if (this.isTiledCrs()) {
        _rowColfromParent(this, extent.zoom);

        return target.set(this.col * r.invDiff - r.col, this.row * r.invDiff - r.row, r.invDiff, r.invDiff);
      }

      extent.dimensions(_dim);
      this.dimensions(_dim2);
      var originX = (this.west - extent.west) / _dim.x;
      var originY = (extent.north - this.north) / _dim.y;
      var scaleX = _dim2.x / _dim.x;
      var scaleY = _dim2.y / _dim.y;
      return target.set(originX, originY, scaleX, scaleY);
    }
    /**
     * Return parent tiled extent with input level
     *
     * @param {number} levelParent level of parent.
     * @return {Extent}
     */

  }, {
    key: "tiledExtentParent",
    value: function tiledExtentParent(levelParent) {
      if (levelParent && levelParent < this.zoom) {
        _rowColfromParent(this, levelParent);

        return new Extent(this.crs, levelParent, r.row, r.col);
      } else {
        return this;
      }
    }
    /**
     * Return true if this bounding box intersect with the bouding box parameter
     * @param {Extent} extent
     * @returns {Boolean}
     */

  }, {
    key: "intersectsExtent",
    value: function intersectsExtent(extent) {
      var other = extent.crs == this.crs ? extent : extent.as(this.crs, _extent);
      return !(this.west >= other.east || this.east <= other.west || this.south >= other.north || this.north <= other.south);
    }
    /**
     * Return the intersection of this extent with another one
     * @param {Extent} extent
     * @returns {Boolean}
     */

  }, {
    key: "intersect",
    value: function intersect(extent) {
      if (!this.intersectsExtent(extent)) {
        return new Extent(this.crs, 0, 0, 0, 0);
      }

      if (extent.crs != this.crs) {
        extent = extent.as(this.crs, _extent);
      }

      return new Extent(this.crs, Math.max(this.west, extent.west), Math.min(this.east, extent.east), Math.max(this.south, extent.south), Math.min(this.north, extent.north));
    }
    /**
     * Set west, east, south and north values.
     * Or if tiled extent, set zoom, row and column values
     * @param {number} v0 west or zoom value
     * @param {number} v1 east or row value
     * @param {number} v2 south or column value
     * @param {number} v3 north value
     * @return {Extent}
     */

  }, {
    key: "set",
    value: function set(v0, v1, v2, v3) {
      if (this.isTiledCrs()) {
        this.zoom = v0;
        this.row = v1;
        this.col = v2;
      } else {
        this._values[CARDINAL.WEST] = v0;
        this._values[CARDINAL.EAST] = v1;
        this._values[CARDINAL.SOUTH] = v2;
        this._values[CARDINAL.NORTH] = v3;
      }

      return this;
    }
    /**
     * Copy to this extent to input extent.
     * @param {Extent} extent
     * @return {Extent} copied extent
     */

  }, {
    key: "copy",
    value: function copy(extent) {
      this.crs = extent.crs;

      if (this.isTiledCrs()) {
        this.set(extent.zoom, extent.row, extent.col);
      } else {
        this.set(extent._values[0], extent._values[1], extent._values[2], extent._values[3]);
      }

      return this;
    }
    /**
     * Union this extent with the input extent.
     * @param {Extent} extent the extent to union.
     */

  }, {
    key: "union",
    value: function union(extent) {
      if (extent.crs != this.crs) {
        throw new Error('unsupported union between 2 diff crs');
      }

      var west = extent.west;

      if (west < this.west) {
        this._values[CARDINAL.WEST] = west;
      }

      var east = extent.east;

      if (east > this.east) {
        this._values[CARDINAL.EAST] = east;
      }

      var south = extent.south;

      if (south < this.south) {
        this._values[CARDINAL.SOUTH] = south;
      }

      var north = extent.north;

      if (north > this.north) {
        this._values[CARDINAL.NORTH] = north;
      }
    }
    /**
     * expandByCoordinates perfoms the minimal extension
     * for the coordinates to belong to this Extent object
     * @param {Coordinates} coordinates  The coordinates to belong
     */

  }, {
    key: "expandByCoordinates",
    value: function expandByCoordinates(coordinates) {
      var coords = coordinates.crs == this.crs ? coordinates : coordinates.as(this.crs, _c);
      this.expandByValuesCoordinates(coords._values[0], coords._values[1]);
    }
    /**
    * expandByValuesCoordinates perfoms the minimal extension
    * for the coordinates values to belong to this Extent object
    * @param {number} we  The coordinate on west-east
    * @param {number} sn  The coordinate on south-north
    *
    */

  }, {
    key: "expandByValuesCoordinates",
    value: function expandByValuesCoordinates(we, sn) {
      if (we < this.west) {
        this._values[CARDINAL.WEST] = we;
      }

      if (we > this.east) {
        this._values[CARDINAL.EAST] = we;
      }

      if (sn < this.south) {
        this._values[CARDINAL.SOUTH] = sn;
      }

      if (sn > this.north) {
        this._values[CARDINAL.NORTH] = sn;
      }
    }
    /**
     * Instance Extent with THREE.Box2
     * @param {string} crs Projection of extent to instancied.
     * @param {THREE.Box2} box
     * @return {Extent}
     */

  }, {
    key: "toString",

    /**
     * Return values of extent in string, separated by the separator input.
     * @param {string} separator
     * @return {string}
     */
    value: function toString() {
      var separator = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      if (this.isTiledCrs()) {
        return "".concat(this.zoom).concat(separator).concat(this.row).concat(separator).concat(this.col);
      } else {
        return "".concat(this.east).concat(separator).concat(this.north).concat(separator).concat(this.west).concat(separator).concat(this.south);
      }
    }
    /**
     * Subdivide equally an extent from its center to return four extents:
     * north-west, north-east, south-west and south-east.
     *
     * @returns {Extent[]} An array containing the four sections of the extent. The
     * order of the sections is [NW, NE, SW, SE].
     */

  }, {
    key: "subdivision",
    value: function subdivision() {
      this.center(_c);
      var northWest = new Extent(this.crs, this.west, _c._values[0], _c._values[1], this.north);
      var northEast = new Extent(this.crs, _c._values[0], this.east, _c._values[1], this.north);
      var southWest = new Extent(this.crs, this.west, _c._values[0], this.south, _c._values[1]);
      var southEast = new Extent(this.crs, _c._values[0], this.east, this.south, _c._values[1]);
      return [northWest, northEast, southWest, southEast];
    }
    /**
     * Apply transform and copy this extent to input.
     * The <code>transformedCopy</code> doesn't handle
     * the issue of overflow of geographic limits.
     * @param {THREE.Vector2} t translation transform
     * @param {THREE.Vector2} s scale transform
     * @param {Extent} extent Extent to copy after transformation.
     */

  }, {
    key: "transformedCopy",
    value: function transformedCopy(t, s, extent) {
      if (!extent.isTiledCrs()) {
        this.crs = extent.crs;
        this._values[0] = (extent._values[0] + t.x) * s.x;
        this._values[1] = (extent._values[1] + t.x) * s.x;

        if (this._values[0] > this._values[1]) {
          var temp = this._values[0];
          this._values[0] = this._values[1];
          this._values[1] = temp;
        }

        this._values[2] = (extent._values[2] + t.y) * s.y;
        this._values[3] = (extent._values[3] + t.y) * s.y;

        if (this._values[2] > this._values[3]) {
          var _temp = this._values[2];
          this._values[2] = this._values[3];
          this._values[3] = _temp;
        }
      }
    }
  }, {
    key: "west",
    get: function get() {
      return this._values[CARDINAL.WEST];
    }
    /**
     * Return the east value
     * @return {number}
     */

  }, {
    key: "east",
    get: function get() {
      return this._values[CARDINAL.EAST];
    }
    /**
     * Return the north value
     * @return {number}
     */

  }, {
    key: "north",
    get: function get() {
      return this._values[CARDINAL.NORTH];
    }
    /**
    * Return the south value
    * @return {number}
    */

  }, {
    key: "south",
    get: function get() {
      return this._values[CARDINAL.SOUTH];
    }
  }], [{
    key: "fromBox3",
    value: function fromBox3(crs, box) {
      return new Extent(crs, {
        west: box.min.x,
        east: box.max.x,
        south: box.min.y,
        north: box.max.y
      });
    }
  }]);
  return Extent;
}();

_extent = new Extent('EPSG:4326', [0, 0, 0, 0]);
var _default = Extent;
exports.default = _default;