"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _three = require("three");

var _Coordinates = _interopRequireDefault(require("./Coordinates"));

var _Extent = _interopRequireDefault(require("./Extent"));

/**
 * Generated On: 2015-10-5
 * Class: Projection
 * Description: Outils de projections cartographiques et de convertion
 */
var PI_OV_FOUR = Math.PI / 4;
var PI_OV_TWO = Math.PI / 2;
var INV_TWO_PI = 1.0 / (Math.PI * 2);
var LOG_TWO = Math.log(2.0);

function WGS84LatitudeClamp(latitude) {
  return Math.min(84, Math.max(-86, latitude));
}

var dim = {
  x: 0,
  y: 0
};
var center = new _Coordinates.default('EPSG:4326', 0, 0, 0);
var Projection = {
  /**
   * Convert latitude to y coordinate in TileMatrixSet
   * @param {number} latitude - latitude in degrees
   * @return {number}
   */
  WGS84ToY: function WGS84ToY(latitude) {
    return 0.5 - Math.log(Math.tan(PI_OV_FOUR + _three.Math.degToRad(latitude) * 0.5)) * INV_TWO_PI;
  },

  /**
   * Convert from y coordinate in TileMatrixSet to WGS84 latitude
   * @param {number} y - coords in TileMatrixSet
   * @return {number} - latitude in degrees
   */
  YToWGS84: function YToWGS84(y) {
    return _three.Math.radToDeg(2 * (Math.atan(Math.exp(-(y - 0.5) / INV_TWO_PI)) - PI_OV_FOUR));
  },
  getCoordWMTS_WGS84: function getCoordWMTS_WGS84(tileCoord, bbox, tileMatrixSet) {
    // TODO: PM, WGS84G are hard-coded reference to IGN's TileMatrixSet
    if (tileMatrixSet === 'PM') {
      return WMTS_WGS84ToWMTS_PM(tileCoord, bbox);
    } else if (tileMatrixSet === 'WGS84G') {
      return [tileCoord.clone()];
    } else {
      throw new Error("Unsupported TileMatrixSet '".concat(tileMatrixSet, "'"));
    }
  },
  WGS84toWMTS: function WGS84toWMTS(bbox) {
    var target = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new _Extent.default('WMTS:WGS84G', 0, 0, 0);
    bbox.dimensions(dim);
    var zoom = Math.floor(Math.log(Math.PI / _three.Math.degToRad(dim.y)) / LOG_TWO + 0.5);
    var nY = Math.pow(2, zoom);
    var uX = Math.PI * 2 / (2 * nY);
    var uY = Math.PI / nY;
    bbox.center(center);
    var col = Math.floor((Math.PI + _three.Math.degToRad(center.longitude())) / uX);
    var row = Math.floor(nY - (PI_OV_TWO + _three.Math.degToRad(center.latitude())) / uY);
    return target.set(zoom, row, col);
  },
  UnitaryToLongitudeWGS84: function UnitaryToLongitudeWGS84(u, bbox) {
    bbox.dimensions(dim);
    return bbox.west + u * dim.x;
  },
  UnitaryToLatitudeWGS84: function UnitaryToLatitudeWGS84(v, bbox) {
    bbox.dimensions(dim);
    return bbox.south + v * dim.y;
  }
};

function WMTS_WGS84ToWMTS_PM(cWMTS, bbox) {
  var wmtsBox = [];
  var level = cWMTS.zoom + 1;
  var nbRow = Math.pow(2, level);
  var sizeRow = 1.0 / nbRow;
  var yMin = Projection.WGS84ToY(WGS84LatitudeClamp(bbox.north));
  var yMax = Projection.WGS84ToY(WGS84LatitudeClamp(bbox.south));
  var maxRow;
  var minRow = Math.floor(yMin / sizeRow); // ]N; N+1] => N

  maxRow = Math.ceil(yMax / sizeRow) - 1; // make sure we don't exceed boundaries

  maxRow = Math.min(maxRow, nbRow - 1);
  var minCol = cWMTS.col;

  for (var r = maxRow; r >= minRow; r--) {
    for (var c = minCol; c <= minCol; c++) {
      wmtsBox.push(new _Extent.default('WMTS:PM', level, r, c));
    }
  }

  return wmtsBox;
}

var _default = Projection;
exports.default = _default;