"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _Coordinates = _interopRequireDefault(require("../../Geographic/Coordinates"));

var _Projection = _interopRequireDefault(require("../../Geographic/Projection"));

var _OBB = _interopRequireDefault(require("../../../Renderer/OBB"));

var _Extent = _interopRequireDefault(require("../../Geographic/Extent"));

var axisZ = new THREE.Vector3(0, 0, 1);
var axisY = new THREE.Vector3(0, 1, 0);

function WGS84ToOneSubY(latitude) {
  return 1.0 - _Projection.default.WGS84ToY(latitude);
}

function BuilderEllipsoidTile() {
  this.tmp = {
    coords: [new _Coordinates.default('EPSG:4326', 0, 0), new _Coordinates.default('EPSG:4326', 0, 0)],
    position: new THREE.Vector3()
  };
  this.type = 'e';
}

BuilderEllipsoidTile.prototype.constructor = BuilderEllipsoidTile; // prepare params
// init projected object -> params.projected

BuilderEllipsoidTile.prototype.Prepare = function (params) {
  params.nbRow = Math.pow(2, params.level + 1.0);
  var st1 = WGS84ToOneSubY(params.extent.south);

  if (!isFinite(st1)) {
    st1 = 0;
  }

  var sizeTexture = 1.0 / params.nbRow;
  var start = st1 % sizeTexture;
  params.deltaUV1 = (st1 - start) * params.nbRow; // transformation to align tile's normal to z axis

  params.quatNormalToZ = new THREE.Quaternion().setFromAxisAngle(axisY, -(Math.PI * 0.5 - THREE.Math.degToRad(params.extent.center().latitude()))); // let's avoid building too much temp objects

  params.projected = {
    longitude: 0,
    latitude: 0
  };
}; // get center tile in cartesian 3D


BuilderEllipsoidTile.prototype.Center = function (extent) {
  return extent.center(this.tmp.coords[0]).as('EPSG:4978', this.tmp.coords[1]).xyz();
}; // get position 3D cartesian


BuilderEllipsoidTile.prototype.VertexPosition = function (params) {
  this.tmp.coords[0].set('EPSG:4326', params.projected.longitude, params.projected.latitude);
  this.tmp.coords[0].as('EPSG:4978', this.tmp.coords[1]).xyz(this.tmp.position);
  return this.tmp.position;
}; // get normal for last vertex


BuilderEllipsoidTile.prototype.VertexNormal = function () {
  return this.tmp.coords[1].geodesicNormal;
}; // coord u tile to projected


BuilderEllipsoidTile.prototype.uProjecte = function (u, params) {
  params.projected.longitude = _Projection.default.UnitaryToLongitudeWGS84(u, params.extent);
}; // coord v tile to projected


BuilderEllipsoidTile.prototype.vProjecte = function (v, params) {
  params.projected.latitude = _Projection.default.UnitaryToLatitudeWGS84(v, params.extent);
}; // Compute uv 1, if isn't defined the uv1 isn't computed


BuilderEllipsoidTile.prototype.getUV_PM = function (params) {
  var t = WGS84ToOneSubY(params.projected.latitude) * params.nbRow;

  if (!isFinite(t)) {
    t = 0;
  }

  return t - params.deltaUV1;
};

var quatToAlignLongitude = new THREE.Quaternion();
var quatToAlignLatitude = new THREE.Quaternion();

BuilderEllipsoidTile.prototype.computeSharableExtent = function (extent) {
  // Compute sharable extent to pool the geometries
  // the geometry in common extent is identical to the existing input
  // with a transformation (translation, rotation)
  // TODO: It should be possible to use equatorial plan symetrie,
  // but we should be reverse UV on tile
  // Common geometry is looking for only on longitude
  var sizeLongitude = Math.abs(extent.west - extent.east) / 2;
  var sharableExtent = new _Extent.default(extent.crs, -sizeLongitude, sizeLongitude, extent.south, extent.north); // compute rotation to transform tile to position it on ellipsoid
  // this transformation take into account the transformation of the parents

  var rotLon = THREE.Math.degToRad(extent.west - sharableExtent.west);
  var rotLat = THREE.Math.degToRad(90 - extent.center(this.tmp.coords[0]).latitude());
  quatToAlignLongitude.setFromAxisAngle(axisZ, rotLon);
  quatToAlignLatitude.setFromAxisAngle(axisY, rotLat);
  quatToAlignLongitude.multiply(quatToAlignLatitude);
  return {
    sharableExtent: sharableExtent,
    quaternion: quatToAlignLongitude.clone(),
    position: this.Center(extent)
  };
}; // use for region for adaptation boundingVolume


BuilderEllipsoidTile.prototype.OBB = function (boundingBox) {
  return new _OBB.default(boundingBox.min, boundingBox.max);
};

var _default = BuilderEllipsoidTile;
exports.default = _default;