"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createPlanarLayer = createPlanarLayer;
exports.default = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _View = _interopRequireDefault(require("../View"));

var _MainLoop = require("../MainLoop");

var _CameraUtils = _interopRequireDefault(require("../../Utils/CameraUtils"));

var _PlanarLayer = _interopRequireDefault(require("./Planar/PlanarLayer"));

function createPlanarLayer(id, extent, options) {
  console.warn('createPlanarLayer is deprecated, use the PlanarLayer class instead.');
  return new _PlanarLayer.default(id, extent, options.object3d, options);
}

function PlanarView(viewerDiv, extent) {
  var _this = this;

  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  THREE.Object3D.DefaultUp.set(0, 0, 1); // Setup View

  _View.default.call(this, extent.crs, viewerDiv, options); // Configure camera


  var dim = extent.dimensions();
  var max = Math.max(dim.x, dim.y);
  var camera3D = this.camera.camera3D;
  camera3D.near = 0.1;
  camera3D.far = 2 * max;
  this.camera.camera3D.updateProjectionMatrix();
  var tileLayer = new _PlanarLayer.default('planar', extent, options.object3d, options);
  this.addLayer(tileLayer);
  var p = {
    coord: extent.center(),
    range: max,
    tilt: 20,
    heading: 0
  };

  _CameraUtils.default.transformCameraToLookAtTarget(this, camera3D, p);

  this._fullSizeDepthBuffer = null;
  this.addFrameRequester(_MainLoop.MAIN_LOOP_EVENTS.BEFORE_RENDER, function () {
    if (_this._fullSizeDepthBuffer != null) {
      // clean depth buffer
      _this._fullSizeDepthBuffer = null;
    }
  });
  this.tileLayer = tileLayer;
}

PlanarView.prototype = Object.create(_View.default.prototype);
PlanarView.prototype.constructor = PlanarView;

PlanarView.prototype.addLayer = function (layer) {
  return _View.default.prototype.addLayer.call(this, layer, this.tileLayer);
};

var _default = PlanarView;
exports.default = _default;