"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.SIZE_DIAGONAL_TEXTURE = exports.SIZE_TEXTURE_TILE = void 0;

var THREE = _interopRequireWildcard(require("three"));

var _Projection = _interopRequireDefault(require("../Core/Geographic/Projection"));

var _Extent = _interopRequireDefault(require("../Core/Geographic/Extent"));

var _Coordinates = _interopRequireDefault(require("../Core/Geographic/Coordinates"));

var c = new _Coordinates.default('EPSG:4326', 180, 85.06);
var layerDimension = new THREE.Vector2();
var tileDimension = new THREE.Vector2(); // Size in pixel

var SIZE_TEXTURE_TILE = 256;
exports.SIZE_TEXTURE_TILE = SIZE_TEXTURE_TILE;
var SIZE_DIAGONAL_TEXTURE = Math.pow(2 * (SIZE_TEXTURE_TILE * SIZE_TEXTURE_TILE), 0.5);
exports.SIZE_DIAGONAL_TEXTURE = SIZE_DIAGONAL_TEXTURE;
var tileCoord = new _Extent.default('WMTS:WGS84G', 0, 0, 0);
var _default = {
  computeTileMatrixSetCoordinates: function computeTileMatrixSetCoordinates(tile, tileMatrixSet) {
    tileMatrixSet = tileMatrixSet || 'WGS84G';

    if (!(tileMatrixSet in tile.wmtsCoords)) {
      if (tile.wmtsCoords.WGS84G) {
        var _c = tile.wmtsCoords.WGS84G[0];
        tileCoord.zoom = _c.zoom;
        tileCoord.col = _c.col;
        tileCoord.row = _c.row;
      } else {
        _Projection.default.WGS84toWMTS(tile.extent, tileCoord);

        tile.wmtsCoords.WGS84G = [tileCoord.clone()];
      }

      tile.wmtsCoords[tileMatrixSet] = _Projection.default.getCoordWMTS_WGS84(tileCoord, tile.extent, tileMatrixSet);
    }
  },
  // The isInverted parameter is to be set to the correct value, true or false
  // (default being false) if the computation of the coordinates needs to be
  // inverted to match the same scheme as OSM, Google Maps or other system.
  // See link below for more information
  // https://alastaira.wordpress.com/2011/07/06/converting-tms-tile-coordinates-to-googlebingosm-tile-coordinates/
  computeTMSCoordinates: function computeTMSCoordinates(tile, extent, isInverted) {
    extent = tile.extent.crs == extent.crs ? extent : extent.as(tile.extent.crs, extent);
    tile.extent.center(c);
    extent.dimensions(layerDimension);
    tile.extent.dimensions(tileDimension); // Each level has 2^n * 2^n tiles...
    // ... so we count how many tiles of the same width as tile we can fit in the layer

    var tileCount = Math.round(layerDimension.x / tileDimension.x); // ... 2^zoom = tilecount => zoom = log2(tilecount)

    var zoom = Math.floor(Math.log2(tileCount)); // Now that we have computed zoom, we can deduce x and y (or row / column)

    var x = (c.x() - extent.west) / layerDimension.x;
    var y;

    if (isInverted) {
      y = (extent.north - c.y()) / layerDimension.y;
    } else {
      y = (c.y() - extent.south) / layerDimension.y;
    }

    return [new _Extent.default('TMS', zoom, Math.floor(y * tileCount), Math.floor(x * tileCount))];
  }
};
exports.default = _default;